
/*********************************************************************/
/*                                                                   */
/* Program name: atop.c                                              */
/*                                                                   */
/* Description: ASCII to Packed Decimal conversion                   */
/*                                                                   */
/*  Statement:  Licensed Materials - Property of IBM                 */
/*                                                                   */
/*              CICS SupportPac CA0A                                 */
/*              (C) Copyright IBM Corp. 2000                         */
/*                                                                   */
/* input  - pointer to target area (char *)                          */
/*          length of target (long)                                  */
/*          pointer to source area (char *)                          */
/*          length of source (long)                                  */
/*                                                                   */
/*********************************************************************/

#include "pdecimal.h"

/* miscellaneous definitions */
#define POSITIVE   0x0c
#define NEGATIVE   0x0d
#define PLUS_SIGN  '+'
#define MINUS_SIGN '-'

int atop( unsigned char *pTarget, long TargetLen,
          unsigned char *pSource, long SourceLen )
{
  short Rc = PDEC_OK;                        /* return code */
  short SIndex;                              /* source index */
  short TIndex;                              /* target index */
  unsigned char Sign = PLUS_SIGN;            /* default sign */

  /* validate source and target lengths */

  if( SourceLen < 1 || SourceLen > 32 )
  {
    Rc = PDEC_BADLENGTH_ASCII;
  }
  else
  {
    if( TargetLen < 1 || TargetLen > 16 )
    {
      Rc = PDEC_BADLENGTH_PACKED;
    }
  }

  if( Rc == PDEC_OK )
  {
    /* clear target */
    memset( pTarget, 0, TargetLen );

    /* get packing - last digit out of the way first */
    pTarget[TargetLen - 1] = pSource[SourceLen - 1] << 4;

    if( SourceLen > 1 && TargetLen > 1 )
    {
      SIndex = SourceLen - 2;
      for( TIndex = TargetLen - 2 ; TIndex >= 0 ; TIndex-- )
      {
        if(( SIndex == 0 ) &&
           ( pSource[0] == PLUS_SIGN || pSource[0] == MINUS_SIGN ))
        {
          Sign = pSource[SIndex];            /* save sign */
          break;                             /* force end of loop */
        }
        else
        {
          pTarget[TIndex] = pSource[SIndex] & 0x0f; /* right-hand nibble */
          if( --SIndex >= 0 )
          {
            pTarget[TIndex] |= pSource[SIndex] << 4; /* left-hand nibble */
            if( --SIndex < 0 )
            {
              break;                         /* force end of loop */
            }
          }
        }
      }
    }

    /* put sign on target */
    if( Sign == PLUS_SIGN )
    {
      pTarget[TargetLen - 1] |= POSITIVE;
    }
    else
    {
      pTarget[TargetLen - 1] |= NEGATIVE;
    }
  }

  return( Rc );
}