
/*********************************************************************/
/*                                                                   */
/* Program name: ltop.c                                              */
/*                                                                   */
/* Description: Long integer to Packed Decimal conversion            */
/*                                                                   */
/*  Statement:  Licensed Materials - Property of IBM                 */
/*                                                                   */
/*              CICS SupportPac CA0A                                 */
/*              (C) Copyright IBM Corp. 2000                         */
/*                                                                   */
/* input  - pointer to target area (char *)                          */
/*          pointer to source area (long *)                          */
/*                                                                   */
/*********************************************************************/

#include "pdecimal.h"

/* miscellaneous definitions */
#define POSITIVE   0x0c
#define NEGATIVE   0x0d
#define K_M        1000000000

int ltop( unsigned char *pTarget, long Source )
{
  short Rc = PDEC_OK;                        /* return code */
  short TargetIndex;
  long  Power = K_M;                         /* Thousands of millions */
  long  LongWork;
  unsigned char CharWork;

  /* The largest positive number a long integer can hold is 2,147,483,647    */
  /* This becomes 00 00 02 14 74 83 64 7C in packed decimal, so we first     */
  /* clear the target, get the thousands of millions figure, then loop round */
  /* the rest, except for the last byte where we have to put the sign on.    */

  memset( pTarget, 0, 8 );                   /* Clear target */

  /* Set sign and use absolute number */
  if( Source < 0 )
  {
    LongWork = Source * -1;
    pTarget[7] = NEGATIVE;
  }
  else
  {
    LongWork = Source;
    pTarget[7] = POSITIVE;
  }

  pTarget[2] = LongWork / Power;             /* Thousands of Millions */
  LongWork -= pTarget[2] * Power;
  Power /= 10;

  for( TargetIndex = 3; TargetIndex < 7; TargetIndex++ )
  {
    CharWork = LongWork / Power;
    pTarget[TargetIndex] = CharWork << 4;
    LongWork -= CharWork * Power;
    Power /= 10;

    CharWork = LongWork / Power;
    pTarget[TargetIndex] |= CharWork;
    LongWork -= CharWork * Power;
    Power /= 10;
  }

  pTarget[7] |= ( LongWork % 10 ) << 4;      /* Last digit */

  return( Rc );
}