
/*********************************************************************/
/*                                                                   */
/* Program name: ptoa.c                                              */
/*                                                                   */
/* Description: Packed Decimal to ASCII conversion                   */
/*                                                                   */
/*  Statement:  Licensed Materials - Property of IBM                 */
/*                                                                   */
/*              CICS SupportPac CA0A                                 */
/*              (C) Copyright IBM Corp. 2000                         */
/*                                                                   */
/* input  - pointer to target area (char *)                          */
/*          length of target (long)                                  */
/*          pointer to source area (char *)                          */
/*          length of source (long)                                  */
/*                                                                   */
/*********************************************************************/

#include "pdecimal.h"

/* miscellaneous definitions */
#define NEGATIVE   0x0d

int ptoa( unsigned char *pTarget, long TargetLen,
          unsigned char *pSource, long SourceLen )
{
  short Rc = PDEC_OK;                        /* return code */
  short SIndex;                              /* source index */
  short TIndex;                              /* target index */

  /* validate source and target lengths */

  if( TargetLen < 1 || TargetLen > 32 )
  {
    Rc = PDEC_BADLENGTH_ASCII;
  }
  else
  {
    if( SourceLen < 1 || SourceLen > 16 )
    {
      Rc = PDEC_BADLENGTH_PACKED;
    }
  }

  if( Rc == PDEC_OK )
  {
    /* clear target */
    memset( pTarget, '0', TargetLen );

    /* get unpacking - last digit out of the way first */
    pTarget[TargetLen - 1] = ( pSource[SourceLen - 1] >> 4 ) | 0x30;

    if( SourceLen > 1 && TargetLen > 1 )
    {
      TIndex = TargetLen - 2;
      for( SIndex = SourceLen - 2 ; SIndex >= 0 ; SIndex-- )
      {
        /* Unpack right half of packed decimal byte */
        pTarget[TIndex] = ( pSource[SIndex] & 0x0f ) | 0x30;
        if( --TIndex >= 0 )
        {
          /* Unpack left half of packed decimal byte */
          pTarget[TIndex] = ( pSource[SIndex] >> 4 ) | 0x30;
          if( --TIndex < 0 )
          {
            break;
          }
        }
        else
        {
          break;                             /* force loop exit */
        }
      }

      if(( Rc == PDEC_OK ) &&
         (( pSource[SourceLen - 1] & 0x0f ) == NEGATIVE ))
      {
        Rc = PDEC_RESULT_NEGATIVE;           /* Return negative sign */
      }
    }
  }

  return( Rc );
}